'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var web3_js = require('@safecoin/web3.js');
var lplCandyMachine = require('@leda-mint-io/lpl-candy-machine');
var program = require('./program.cjs');
var helpers = require('./helpers.cjs');
var CandyMachine = require('./CandyMachine.cjs');
var pdas$1 = require('./pdas.cjs');
var errors = require('./errors.cjs');
var Nft = require('../nftModule/Nft.cjs');
var Creator = require('../../types/Creator.cjs');
var Amount = require('../../types/Amount.cjs');
var pdas = require('../nftModule/pdas.cjs');
var program$1 = require('../nftModule/program.cjs');
var Operation = require('../../types/Operation.cjs');
var BigNumber = require('../../types/BigNumber.cjs');
var PublicKey = require('../../types/PublicKey.cjs');
var TransactionBuilder = require('../../utils/TransactionBuilder.cjs');
var Signer = require('../../types/Signer.cjs');

// Operation
// -----------------

const Key = 'CreateCandyMachineOperation';
const createCandyMachineOperation = Operation.useOperation(Key);
// -----------------
// Handler
// -----------------
const createCandyMachineOperationHandler = {
  async handle(operation, metaplex, scope) {
    const builder = await createCandyMachineBuilder(metaplex, operation.input);
    scope.throwIfCanceled();
    return builder.sendAndConfirm(metaplex, operation.input.confirmOptions);
  }

}; // -----------------
// Builder
// -----------------

const createCandyMachineBuilder = async (metaplex, params) => {
  const candyMachine = params.candyMachine ?? web3_js.Keypair.generate();
  const payer = params.payer ?? metaplex.identity();
  const authority = params.authority ?? metaplex.identity();
  const collection = params.collection && (Nft.isNft(params.collection) || Nft.isLazyNft(params.collection)) ? params.collection.mintAddress : params.collection ?? null;
  const {
    data,
    wallet,
    tokenMint
  } = CandyMachine.toCandyMachineInstructionData(candyMachine.publicKey, { ...params,
    wallet: params.wallet ?? metaplex.identity().publicKey,
    tokenMint: params.tokenMint ?? null,
    symbol: params.symbol ?? '',
    maxEditionSupply: params.maxEditionSupply ?? BigNumber.toBigNumber(0),
    isMutable: params.isMutable ?? true,
    retainAuthority: params.retainAuthority ?? true,
    goLiveDate: params.goLiveDate ?? null,
    endSettings: params.endSettings ?? null,
    creators: params.creators ?? Creator.toUniformCreators(metaplex.identity().publicKey),
    hiddenSettings: params.hiddenSettings ?? null,
    whitelistMintSettings: params.whitelistMintSettings ?? null,
    gatekeeper: params.gatekeeper ?? null
  });
  const initializeInstruction = lplCandyMachine.createInitializeCandyMachineInstruction({
    candyMachine: candyMachine.publicKey,
    wallet,
    authority: PublicKey.toPublicKey(authority),
    payer: payer.publicKey
  }, {
    data
  });

  if (tokenMint) {
    initializeInstruction.keys.push({
      pubkey: tokenMint,
      isWritable: false,
      isSigner: false
    });
  } else {
    Amount.assertSameCurrencies(params.price, Amount.SOL);
  }

  return TransactionBuilder.TransactionBuilder.make().setFeePayer(payer).setContext({
    candyMachineSigner: candyMachine,
    payer,
    wallet,
    authority: PublicKey.toPublicKey(authority),
    creators: data.creators
  }) // Create an empty account for the candy machine.
  .add(await metaplex.system().builders().createAccount({
    payer,
    newAccount: candyMachine,
    space: helpers.getCandyMachineAccountSizeFromData(data),
    program: program.CandyMachineProgram.publicKey,
    instructionKey: params.createAccountInstructionKey ?? 'createAccount'
  })) // Initialize the candy machine account.
  .add({
    instruction: initializeInstruction,
    signers: [candyMachine, payer],
    key: params.initializeCandyMachineInstructionKey ?? 'initializeCandyMachine'
  }) // Set the collection.
  .when(!!collection, builder => {
    if (!Signer.isSigner(authority)) {
      throw new errors.CandyMachineAuthorityRequiredAsASignerError();
    }

    const collectionMint = collection;
    const metadata = pdas.findMetadataPda(collectionMint);
    const edition = pdas.findMasterEditionV2Pda(collectionMint);
    const collectionPda = pdas$1.findCandyMachineCollectionPda(candyMachine.publicKey);
    const collectionAuthorityRecord = pdas.findCollectionAuthorityRecordPda(collectionMint, collectionPda);
    return builder.add({
      instruction: lplCandyMachine.createSetCollectionInstruction({
        candyMachine: candyMachine.publicKey,
        authority: PublicKey.toPublicKey(authority),
        collectionPda,
        payer: payer.publicKey,
        metadata,
        mint: collectionMint,
        edition,
        collectionAuthorityRecord,
        tokenMetadataProgram: program$1.TokenMetadataProgram.publicKey
      }),
      signers: [authority],
      key: params.setCollectionInstructionKey ?? 'setCollection'
    });
  });
};

exports.createCandyMachineBuilder = createCandyMachineBuilder;
exports.createCandyMachineOperation = createCandyMachineOperation;
exports.createCandyMachineOperationHandler = createCandyMachineOperationHandler;
//# sourceMappingURL=createCandyMachine.cjs.map
