'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var web3_js = require('@safecoin/web3.js');
var nacl = require('tweetnacl');
var buffer = require('buffer');
var errors = require('./errors.cjs');
var Task = require('../../utils/Task.cjs');
var Signer = require('../../types/Signer.cjs');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var nacl__default = /*#__PURE__*/_interopDefaultLegacy(nacl);

class DerivedIdentityClient {
  originalSigner = null;
  derivedKeypair = null;

  constructor(metaplex) {
    this.metaplex = metaplex;
  }

  get publicKey() {
    this.assertInitialized();
    return this.derivedKeypair.publicKey;
  }

  get secretKey() {
    this.assertInitialized();
    return this.derivedKeypair.secretKey;
  }

  get originalPublicKey() {
    this.assertInitialized();
    return this.originalSigner.publicKey;
  }

  deriveFrom(message, originalSigner) {
    return new Task.Task(async () => {
      this.originalSigner = originalSigner ?? this.metaplex.identity().driver();
      const signature = await this.originalSigner.signMessage(buffer.Buffer.from(message));
      const seeds = nacl__default["default"].hash(signature).slice(0, 32);
      this.derivedKeypair = web3_js.Keypair.fromSeed(seeds);
    });
  }

  fund(amount) {
    this.assertInitialized();
    return this.metaplex.system().transferSol({
      from: this.originalSigner,
      to: this.derivedKeypair.publicKey,
      amount
    });
  }

  withdraw(amount) {
    this.assertInitialized();
    return this.metaplex.system().transferSol({
      from: this.derivedKeypair,
      to: this.originalSigner.publicKey,
      amount
    });
  }

  withdrawAll() {
    this.assertInitialized();
    return new Task.Task(async scope => {
      this.assertInitialized();
      const balance = await this.metaplex.rpc().getBalance(this.derivedKeypair.publicKey);
      return this.withdraw(balance).run(scope);
    });
  }

  close() {
    this.originalSigner = null;
    this.derivedKeypair = null;
  }

  async signMessage(message) {
    return nacl__default["default"].sign.detached(message, this.secretKey);
  }

  async signTransaction(transaction) {
    transaction.partialSign(this);
    return transaction;
  }

  async signAllTransactions(transactions) {
    return Promise.all(transactions.map(transaction => this.signTransaction(transaction)));
  }

  verifyMessage(message, signature) {
    return nacl__default["default"].sign.detached.verify(message, signature, this.publicKey.toBytes());
  }

  equals(that) {
    if (Signer.isSigner(that)) {
      that = that.publicKey;
    }

    return this.publicKey.equals(that);
  }

  assertInitialized() {
    if (this.derivedKeypair === null || this.originalSigner === null) {
      throw new errors.UninitializedDerivedIdentityError();
    }
  }

}

exports.DerivedIdentityClient = DerivedIdentityClient;
//# sourceMappingURL=DerivedIdentityClient.cjs.map
