'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var web3_js = require('@safecoin/web3.js');
var Signer = require('../../types/Signer.cjs');
var RpcError = require('../../errors/RpcError.cjs');
var Amount = require('../../types/Amount.cjs');
var Program = require('../../types/Program.cjs');
var ProgramError = require('../../errors/ProgramError.cjs');
var TransactionBuilder = require('../../utils/TransactionBuilder.cjs');
var common = require('../../utils/common.cjs');

class RpcClient {
  constructor(metaplex) {
    this.metaplex = metaplex;
  }

  async sendTransaction(transaction, signers = [], sendOptions = {}) {
    var _transaction, _transaction2;

    if (transaction instanceof TransactionBuilder.TransactionBuilder) {
      signers = [...transaction.getSigners(), ...signers];
      transaction = transaction.toTransaction();
    }

    (_transaction = transaction).feePayer ?? (_transaction.feePayer = this.getDefaultFeePayer());
    (_transaction2 = transaction).recentBlockhash ?? (_transaction2.recentBlockhash = await this.getLatestBlockhash());

    if (transaction.feePayer && this.metaplex.identity().equals(transaction.feePayer)) {
      signers = [this.metaplex.identity(), ...signers];
    }

    const {
      keypairs,
      identities
    } = Signer.getSignerHistogram(signers);

    if (keypairs.length > 0) {
      transaction.partialSign(...keypairs);
    }

    for (let i = 0; i < identities.length; i++) {
      await identities[i].signTransaction(transaction);
    }

    const rawTransaction = transaction.serialize();

    try {
      return await this.metaplex.connection.sendRawTransaction(rawTransaction, sendOptions);
    } catch (error) {
      throw this.parseProgramError(error, transaction);
    }
  }

  async confirmTransaction(signature, commitment) {
    let rpcResponse;

    try {
      rpcResponse = await this.metaplex.connection.confirmTransaction(signature, commitment);
    } catch (error) {
      throw new RpcError.FailedToConfirmTransactionError(error);
    }

    if (rpcResponse.value.err) {
      throw new RpcError.FailedToConfirmTransactionWithResponseError(rpcResponse);
    }

    return rpcResponse;
  }

  async sendAndConfirmTransaction(transaction, signers, confirmOptions) {
    const signature = await this.sendTransaction(transaction, signers, confirmOptions);
    const confirmResponse = await this.confirmTransaction(signature, confirmOptions?.commitment);
    return {
      signature,
      confirmResponse
    };
  }

  async getAccount(publicKey, commitment) {
    const accountInfo = await this.metaplex.connection.getAccountInfo(publicKey, commitment);
    return this.getUnparsedMaybeAccount(publicKey, accountInfo);
  }

  async getMultipleAccounts(publicKeys, commitment) {
    const accountInfos = await this.metaplex.connection.getMultipleAccountsInfo(publicKeys, commitment);
    return common.zipMap(publicKeys, accountInfos, (publicKey, accountInfo) => {
      return this.getUnparsedMaybeAccount(publicKey, accountInfo);
    });
  }

  async getProgramAccounts(programId, configOrCommitment) {
    const accounts = await this.metaplex.connection.getProgramAccounts(programId, configOrCommitment);
    return accounts.map(({
      pubkey,
      account
    }) => ({
      publicKey: pubkey,
      ...account
    }));
  }

  async airdrop(publicKey, amount, commitment) {
    Amount.assertSol(amount);
    const signature = await this.metaplex.connection.requestAirdrop(publicKey, amount.basisPoints.toNumber());
    const confirmResponse = await this.confirmTransaction(signature, commitment);
    return {
      signature,
      confirmResponse
    };
  }

  async getBalance(publicKey, commitment) {
    const balance = await this.metaplex.connection.getBalance(publicKey, commitment);
    return Amount.lamports(balance);
  }

  async getRent(bytes, commitment) {
    const rent = await this.metaplex.connection.getMinimumBalanceForRentExemption(bytes, commitment);
    return Amount.lamports(rent);
  }

  async getLatestBlockhash() {
    return (await this.metaplex.connection.getLatestBlockhash('finalized')).blockhash;
  }

  getSolanaExporerUrl(signature) {
    let clusterParam = '';

    switch (this.metaplex.cluster) {
      case 'devnet':
        clusterParam = '?cluster=devnet';
        break;

      case 'testnet':
        clusterParam = '?cluster=testnet';
        break;

      case 'localnet':
      case 'custom':
        const url = encodeURIComponent(this.metaplex.connection.rpcEndpoint);
        clusterParam = `?cluster=custom&customUrl=${url}`;
        break;
    }

    return `https://explorer.solana.com/tx/${signature}${clusterParam}`;
  }

  getDefaultFeePayer() {
    const identity = this.metaplex.identity().publicKey;
    return identity.equals(web3_js.PublicKey.default) ? undefined : identity;
  }

  getUnparsedMaybeAccount(publicKey, accountInfo) {
    if (!accountInfo) {
      return {
        publicKey,
        exists: false
      };
    }

    return {
      publicKey,
      exists: true,
      ...accountInfo
    };
  }

  parseProgramError(error, transaction) {
    // Ensure the error as logs.
    if (!Program.isErrorWithLogs(error)) {
      return new RpcError.FailedToSendTransactionError(error);
    } // Parse the instruction number.


    const regex = /Error processing Instruction (\d+):/;
    const instruction = error.message.match(regex)?.[1] ?? null; // Ensure there is an instruction number given to find the program.

    if (!instruction) {
      return new RpcError.FailedToSendTransactionError(error);
    } // Get the program ID from the instruction in the transaction.


    const instructionNumber = parseInt(instruction, 10);
    const programId = transaction.instructions?.[instructionNumber]?.programId ?? null; // Ensure we were able to find a program ID for the instruction.

    if (!programId) {
      return new RpcError.FailedToSendTransactionError(error);
    } // Find a registered program if any.


    let program;

    try {
      program = this.metaplex.programs().get(programId);
    } catch (_programNotFoundError) {
      return new RpcError.FailedToSendTransactionError(error);
    } // Ensure an error resolver exists on the program.


    if (!program.errorResolver) {
      return new ProgramError.UnknownProgramError(program, error);
    } // Finally, resolve the error.


    const resolvedError = program.errorResolver(error);
    return resolvedError ? new ProgramError.ParsedProgramError(program, resolvedError) : new ProgramError.UnknownProgramError(program, error);
  }

}

exports.RpcClient = RpcClient;
//# sourceMappingURL=RpcClient.cjs.map
