import { SYSVAR_INSTRUCTIONS_PUBKEY } from '@safecoin/web3.js';
import { createAuctioneerSellInstruction, createSellInstruction, createPrintListingReceiptInstruction } from '@leda-mint-io/lpl-auction-house';
import { findAuctionHouseTradeStatePda, findAuctionHouseProgramAsSignerPda, findAuctioneerPda, findListingReceiptPda } from './pdas.mjs';
import { AUCTIONEER_PRICE } from './constants.mjs';
import { useOperation } from '../../types/Operation.mjs';
import { token, lamports, amount } from '../../types/Amount.mjs';
import { findMetadataPda } from '../nftModule/pdas.mjs';
import { findAssociatedTokenAccountPda } from '../tokenModule/pdas.mjs';
import { toPublicKey } from '../../types/PublicKey.mjs';
import { isSigner } from '../../types/Signer.mjs';
import { TransactionBuilder } from '../../utils/TransactionBuilder.mjs';

// Operation
// -----------------

const Key = 'CreateListingOperation';
const createListingOperation = useOperation(Key);
// -----------------
// Handler
// -----------------
const createListingOperationHandler = {
  handle: async (operation, metaplex) => {
    return createListingBuilder(metaplex, operation.input).sendAndConfirm(metaplex, operation.input.confirmOptions);
  }
}; // -----------------
// Builder
// -----------------

const createListingBuilder = (metaplex, params) => {
  // Data.
  const auctionHouse = params.auctionHouse;
  const tokens = params.tokens ?? token(1);
  const priceBasisPoint = params.auctioneerAuthority ? AUCTIONEER_PRICE : params.price?.basisPoints ?? 0;
  const price = auctionHouse.isNative ? lamports(priceBasisPoint) : amount(priceBasisPoint, auctionHouse.treasuryMint.currency); // Accounts.

  const seller = params.seller ?? metaplex.identity();
  const authority = params.authority ?? auctionHouse.authorityAddress;
  const metadata = findMetadataPda(params.mintAccount);
  const tokenAccount = params.tokenAccount ?? findAssociatedTokenAccountPda(params.mintAccount, toPublicKey(seller));
  const sellerTradeState = findAuctionHouseTradeStatePda(auctionHouse.address, toPublicKey(seller), auctionHouse.treasuryMint.address, params.mintAccount, price.basisPoints, tokens.basisPoints, tokenAccount);
  const freeSellerTradeState = findAuctionHouseTradeStatePda(auctionHouse.address, toPublicKey(seller), auctionHouse.treasuryMint.address, params.mintAccount, lamports(0).basisPoints, tokens.basisPoints, tokenAccount);
  const programAsSigner = findAuctionHouseProgramAsSignerPda();
  const accounts = {
    wallet: toPublicKey(seller),
    tokenAccount,
    metadata,
    authority: toPublicKey(authority),
    auctionHouse: auctionHouse.address,
    auctionHouseFeeAccount: auctionHouse.feeAccountAddress,
    sellerTradeState,
    freeSellerTradeState,
    programAsSigner
  }; // Args.

  const args = {
    tradeStateBump: sellerTradeState.bump,
    freeTradeStateBump: freeSellerTradeState.bump,
    programAsSignerBump: programAsSigner.bump,
    buyerPrice: price.basisPoints,
    tokenSize: tokens.basisPoints
  }; // Sell Instruction.

  let sellInstruction;

  if (params.auctioneerAuthority) {
    sellInstruction = createAuctioneerSellInstruction({ ...accounts,
      auctioneerAuthority: params.auctioneerAuthority.publicKey,
      ahAuctioneerPda: findAuctioneerPda(auctionHouse.address, params.auctioneerAuthority.publicKey)
    }, args);
  } else {
    sellInstruction = createSellInstruction(accounts, args);
  } // Signers.


  const sellSigners = [seller, authority, params.auctioneerAuthority].filter(input => !!input && isSigner(input)); // Receipt.

  const bookkeeper = params.bookkeeper ?? metaplex.identity();
  const receipt = findListingReceiptPda(sellerTradeState);
  return TransactionBuilder.make().setContext({
    sellerTradeState,
    freeSellerTradeState,
    tokenAccount,
    metadata,
    seller: toPublicKey(seller),
    receipt,
    bookkeeper: bookkeeper.publicKey,
    price,
    tokens
  }) // Create Listing.
  .add({
    instruction: sellInstruction,
    signers: sellSigners,
    key: 'sell'
  }) // Print the Listing Receipt.
  .when(params.printReceipt ?? true, builder => builder.add({
    instruction: createPrintListingReceiptInstruction({
      receipt,
      bookkeeper: bookkeeper.publicKey,
      instruction: SYSVAR_INSTRUCTIONS_PUBKEY
    }, {
      receiptBump: receipt.bump
    }),
    signers: [bookkeeper],
    key: 'printListingReceipt'
  }));
};

export { createListingBuilder, createListingOperation, createListingOperationHandler };
//# sourceMappingURL=createListing.mjs.map
