import { Keypair } from '@safecoin/web3.js';
import { createInitializeCandyMachineInstruction, createSetCollectionInstruction } from '@leda-mint-io/lpl-candy-machine';
import { CandyMachineProgram } from './program.mjs';
import { getCandyMachineAccountSizeFromData } from './helpers.mjs';
import { toCandyMachineInstructionData } from './CandyMachine.mjs';
import { findCandyMachineCollectionPda } from './pdas.mjs';
import { CandyMachineAuthorityRequiredAsASignerError } from './errors.mjs';
import { isNft, isLazyNft } from '../nftModule/Nft.mjs';
import { toUniformCreators } from '../../types/Creator.mjs';
import { assertSameCurrencies, SOL } from '../../types/Amount.mjs';
import { findMetadataPda, findMasterEditionV2Pda, findCollectionAuthorityRecordPda } from '../nftModule/pdas.mjs';
import { TokenMetadataProgram } from '../nftModule/program.mjs';
import { useOperation } from '../../types/Operation.mjs';
import { toBigNumber } from '../../types/BigNumber.mjs';
import { toPublicKey } from '../../types/PublicKey.mjs';
import { TransactionBuilder } from '../../utils/TransactionBuilder.mjs';
import { isSigner } from '../../types/Signer.mjs';

// Operation
// -----------------

const Key = 'CreateCandyMachineOperation';
const createCandyMachineOperation = useOperation(Key);
// -----------------
// Handler
// -----------------
const createCandyMachineOperationHandler = {
  async handle(operation, metaplex, scope) {
    const builder = await createCandyMachineBuilder(metaplex, operation.input);
    scope.throwIfCanceled();
    return builder.sendAndConfirm(metaplex, operation.input.confirmOptions);
  }

}; // -----------------
// Builder
// -----------------

const createCandyMachineBuilder = async (metaplex, params) => {
  const candyMachine = params.candyMachine ?? Keypair.generate();
  const payer = params.payer ?? metaplex.identity();
  const authority = params.authority ?? metaplex.identity();
  const collection = params.collection && (isNft(params.collection) || isLazyNft(params.collection)) ? params.collection.mintAddress : params.collection ?? null;
  const {
    data,
    wallet,
    tokenMint
  } = toCandyMachineInstructionData(candyMachine.publicKey, { ...params,
    wallet: params.wallet ?? metaplex.identity().publicKey,
    tokenMint: params.tokenMint ?? null,
    symbol: params.symbol ?? '',
    maxEditionSupply: params.maxEditionSupply ?? toBigNumber(0),
    isMutable: params.isMutable ?? true,
    retainAuthority: params.retainAuthority ?? true,
    goLiveDate: params.goLiveDate ?? null,
    endSettings: params.endSettings ?? null,
    creators: params.creators ?? toUniformCreators(metaplex.identity().publicKey),
    hiddenSettings: params.hiddenSettings ?? null,
    whitelistMintSettings: params.whitelistMintSettings ?? null,
    gatekeeper: params.gatekeeper ?? null
  });
  const initializeInstruction = createInitializeCandyMachineInstruction({
    candyMachine: candyMachine.publicKey,
    wallet,
    authority: toPublicKey(authority),
    payer: payer.publicKey
  }, {
    data
  });

  if (tokenMint) {
    initializeInstruction.keys.push({
      pubkey: tokenMint,
      isWritable: false,
      isSigner: false
    });
  } else {
    assertSameCurrencies(params.price, SOL);
  }

  return TransactionBuilder.make().setFeePayer(payer).setContext({
    candyMachineSigner: candyMachine,
    payer,
    wallet,
    authority: toPublicKey(authority),
    creators: data.creators
  }) // Create an empty account for the candy machine.
  .add(await metaplex.system().builders().createAccount({
    payer,
    newAccount: candyMachine,
    space: getCandyMachineAccountSizeFromData(data),
    program: CandyMachineProgram.publicKey,
    instructionKey: params.createAccountInstructionKey ?? 'createAccount'
  })) // Initialize the candy machine account.
  .add({
    instruction: initializeInstruction,
    signers: [candyMachine, payer],
    key: params.initializeCandyMachineInstructionKey ?? 'initializeCandyMachine'
  }) // Set the collection.
  .when(!!collection, builder => {
    if (!isSigner(authority)) {
      throw new CandyMachineAuthorityRequiredAsASignerError();
    }

    const collectionMint = collection;
    const metadata = findMetadataPda(collectionMint);
    const edition = findMasterEditionV2Pda(collectionMint);
    const collectionPda = findCandyMachineCollectionPda(candyMachine.publicKey);
    const collectionAuthorityRecord = findCollectionAuthorityRecordPda(collectionMint, collectionPda);
    return builder.add({
      instruction: createSetCollectionInstruction({
        candyMachine: candyMachine.publicKey,
        authority: toPublicKey(authority),
        collectionPda,
        payer: payer.publicKey,
        metadata,
        mint: collectionMint,
        edition,
        collectionAuthorityRecord,
        tokenMetadataProgram: TokenMetadataProgram.publicKey
      }),
      signers: [authority],
      key: params.setCollectionInstructionKey ?? 'setCollection'
    });
  });
};

export { createCandyMachineBuilder, createCandyMachineOperation, createCandyMachineOperationHandler };
//# sourceMappingURL=createCandyMachine.mjs.map
